%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% DemoMullersMethodSimple
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Overview:
%   Calculate the roots of a complex-valued function using
%   Muller's method.
%
% References:
%   Section 1.6
%   Mathematical and Computational Methods in Photonics and Phononics
%
% Authors:
%   Habib Ammari, Brian Fitzpatrick, Hyeonbae Kang,
%   Matias Ruiz, Sanghyeon Yu, Hai Zhang
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc
clear all
close all
% --------------------------------------------------
% Define the complex function f(z) = sin(z) + 5 +  1i which has
% roots z = 2*pi*n - sin^-1(5+i) or 2*pi*n + pi + sin^-1(5+i)
% for n an integer.
% --------------------------------------------------
f = @(z) sin(z) + 5 +  1i;
z0_exact = - asin(5 + 1i);

% --------------------------------------------------
% Determine the comlex root using Muller's method.
% --------------------------------------------------
z0 = -1.2;
z1 = -1.4;
z2 = -1.5;

% --------------------------------------------------
% Set tolerances
% --------------------------------------------------
distTol = 5e-5;
fTol = 1e-12;
iterMax = 100;
[z0_muller, fRoot, iterN] = MullersMethod(f, z0, z1, z2, iterMax, distTol, fTol);

fprintf('Exact root:               %.8f %.8fi\n', real(z0_exact), imag(z0_exact));
fprintf('Muller method root:       %.8f %.8fi\n', real(z0_muller), imag(z0_muller));
fprintf('Number of iterations:     %d\n', iterN);